interface ScalerInterface {
    fit(X: number[][]): void;
    transform(X: number[][]): number[][];
    fitTransform(X: number[][]): number[][];
}
interface ModelInterface {
    fit(X: number[][], y: number[]): void;
    predict(X: number[][]): number[];
    score(X: number[][], y: number[]): number;
}

/**
 * Linear Regression model implementation using ordinary least squares
 * @class LinearRegression
 * @implements {ModelInterface}
 */
declare class LinearRegression implements ModelInterface {
    private weights;
    /**
     * Fits the linear regression model to the training data
     * @param {number[][]} X - Training data features matrix
     * @param {number[]} y - Training data target values
     * @throws {DataValidationError} If input dimensions don't match or data is empty
     */
    fit(X: number[][], y: number[]): void;
    /**
     * Make predictions using the trained model
     * @param X Prediction data matrix, each row represents a sample, each column represents a feature
     * @returns Vector of predictions
     * @throws {DataValidationError} If model is not trained or input dimensions are incorrect
     */
    predict(X: number[][]): number[];
    /**
     * Solve linear system Ax = b using SVD
     * @param {math.Matrix} A - Coefficient matrix
     * @param {math.Matrix} b - Target values matrix
     * @returns {math.Matrix} Solution matrix
     */
    private solve;
    /**
     * Calculate the R² score of the model
     * @param {number[][]} X - Prediction data matrix
     * @param {number[]} y - Target values
     * @returns {number} R² score
     */
    score(X: number[][], y: number[]): number;
}

/**
 * Stochastic Gradient Descent Regressor
 * @class SGDRegressor
 * @implements {ModelInterface}
 */
declare class SGDRegressor implements ModelInterface {
    private weights;
    private learningRate;
    private epochs;
    private lossFunction;
    constructor(learningRate?: number, epochs?: number, lossFunction?: string);
    /**
     * Fits the model to the training data
     * @param {number[][]} X - Training data features matrix
     * @param {number[]} y - Training data target values
     */
    fit(X: number[][], y: number[]): void;
    /**
     * Predicts the target values for the given data
     * @param {number[][]} X - Data features matrix
     * @returns {number[]} Predicted target values
     */
    predict(X: number[][]): number[];
    /**
     * Evaluates the model's performance
     * @param {number[][]} X - Data features matrix
     * @param {number[]} y - Target values
     * @returns {number} R-squared score
     */
    score(X: number[][], y: number[]): number;
    /**
     * Calculates the dot product of two vectors
     * @param {number[]} a - First vector
     * @param {number[]} b - Second vector
     * @returns {number} Dot product
     */
    private dotProduct;
}

declare class StandardScaler implements ScalerInterface {
    private mean;
    private std;
    /**
     * Calculate the mean and standard deviation of features
     * @param {number[][]} X - Input data matrix
     * @throws {Error} If input array is empty or has inconsistent feature dimensions
     */
    fit(X: number[][]): void;
    /**
     * Standardize data using the calculated mean and standard deviation
     * @param {number[][]} X - Input data matrix
     * @returns {number[][]} Standardized data matrix
     */
    transform(X: number[][]): number[][];
    /**
     * Combine fit and transform operations
     * @param {number[][]} X - Input data matrix
     * @returns {number[][]} Standardized data matrix
     */
    fitTransform(X: number[][]): number[][];
}

declare class MinMaxScaler implements ScalerInterface {
    private min;
    private max;
    /**
     * Fit the scaler to the data
     * @param {number[][]} X - Input data matrix
     */
    fit(X: number[][]): void;
    /**
     * Fit the scaler to the data and transform it
     * @param X Input data matrix
     * @returns Scaled data matrix
     */
    fitTransform(X: number[][]): number[][];
    /**
     * Transform data using the fitted scaler
     * @param X Input data matrix
     * @returns Scaled data matrix
     */
    transform(X: number[][]): number[][];
}

/**
 * Split data into training and testing sets
 * @param {number} testSize - Proportion of data to include in the test set
 * @param {number} [randomSeed] - Optional random seed for reproducibility
 * @param {...any[][]} dataArrays - Input data arrays
 * @returns {Array<{train: any[], test: any[]}>} Array of objects containing training and testing sets
 */
declare function trainTestSplit<T>(testSize?: number, randomSeed?: number, ...dataArrays: any[][]): Array<{
    train: any[];
    test: any[];
}>;

interface PipelineResult {
    model: LinearRegression | SGDRegressor;
    predictions: number[];
    trueValues: number[];
    scaler?: StandardScaler | MinMaxScaler;
    testFeaturesRaw?: Dataset;
    testFeaturesScaled?: Dataset;
    trainingScore?: number;
}
type Dataset = number[][];

/**
 * Creates and trains a machine learning pipeline with data preprocessing and model training
 * @param X Feature matrix
 * @param y Target vector
 * @param testSize Proportion of data to use for testing (default: 0.2)
 * @returns PipelineResult containing trained model, predictions and true values
 */
declare function createPipelineAndTrain(X: Dataset, y: number[], testSize?: number, modelType?: "linear" | "sgd", preprocess?: "standard" | "minmax", randomSeed?: number): PipelineResult;

export { type Dataset, LinearRegression, MinMaxScaler, type PipelineResult, SGDRegressor, StandardScaler, createPipelineAndTrain, trainTestSplit };
