"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  LinearRegression: () => LinearRegression,
  MinMaxScaler: () => MinMaxScaler,
  SGDRegressor: () => SGDRegressor,
  StandardScaler: () => StandardScaler,
  createPipelineAndTrain: () => createPipelineAndTrain,
  trainTestSplit: () => trainTestSplit
});
module.exports = __toCommonJS(src_exports);

// src/models/LinearRegression.ts
var math = __toESM(require("mathjs"));
var import_numeric = __toESM(require("numeric"));

// src/exceptions/index.ts
var DataValidationError = class extends Error {
  constructor(message) {
    super(message);
    this.name = "DataValidationError";
  }
};
var SGDRegressorError = class extends Error {
  constructor(message) {
    super(message);
    this.name = "SGDRegressorError";
  }
};

// src/models/LinearRegression.ts
var LinearRegression = class {
  constructor() {
    this.weights = null;
  }
  /**
   * Fits the linear regression model to the training data
   * @param {number[][]} X - Training data features matrix
   * @param {number[]} y - Training data target values
   * @throws {DataValidationError} If input dimensions don't match or data is empty
   */
  fit(X, y) {
    if (X.length !== y.length) {
      throw new DataValidationError(
        `Number of samples in X(${X.length}) and y(${y.length}) must match`
      );
    }
    if (X.length === 0) {
      throw new DataValidationError("Empty training data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new DataValidationError("Inconsistent number of features in X");
    }
    const X_transpose = math.transpose(math.matrix(X));
    const XTX = math.multiply(X_transpose, X);
    const XTy = math.multiply(X_transpose, math.matrix(y.map((val) => [val])));
    this.weights = this.solve(XTX, XTy);
  }
  /**
   * Make predictions using the trained model
   * @param X Prediction data matrix, each row represents a sample, each column represents a feature
   * @returns Vector of predictions
   * @throws {DataValidationError} If model is not trained or input dimensions are incorrect
   */
  predict(X) {
    if (!this.weights) {
      throw new DataValidationError("Model not trained yet");
    }
    if (X.length === 0) {
      throw new DataValidationError("Empty prediction data");
    }
    if (!X.every((row) => row.length === this.weights.size()[0])) {
      throw new DataValidationError(
        `Number of features(${X[0].length}) must match training data(${this.weights.size()[0]})`
      );
    }
    return X.map((row) => math.dot(row, this.weights.toArray()));
  }
  /**
   * Solve linear system Ax = b using SVD
   * @param {math.Matrix} A - Coefficient matrix
   * @param {math.Matrix} b - Target values matrix
   * @returns {math.Matrix} Solution matrix
   */
  solve(A, b) {
    const { U, S, V } = import_numeric.default.svd(A.toArray());
    const S_diag = math.matrix(
      math.zeros(A.size()[0], A.size()[1])
    );
    for (let i = 0; i < S.length; i++) {
      S_diag.set([i, i], S[i]);
    }
    const S_end = math.matrix(S_diag);
    const S_inv = math.inv(S_end);
    const V_T = math.transpose(V);
    const X = math.multiply(V_T, math.multiply(S_inv, math.multiply(U, b)));
    return X;
  }
  /**
   * Calculate the R² score of the model
   * @param {number[][]} X - Prediction data matrix
   * @param {number[]} y - Target values
   * @returns {number} R² score
   */
  score(X, y) {
    const predictions = this.predict(X);
    const meanY = y.reduce((sum, val) => sum + val, 0) / y.length;
    let totalSS = 0;
    let residualSS = 0;
    for (let i = 0; i < y.length; i++) {
      totalSS += Math.pow(y[i] - meanY, 2);
      residualSS += Math.pow(y[i] - predictions[i], 2);
    }
    if (totalSS === 0) return 0;
    return 1 - residualSS / totalSS;
  }
};

// src/models/SGDRegressor.ts
var SGDRegressor = class {
  constructor(learningRate = 0.01, epochs = 1e3, lossFunction = "mse") {
    this.weights = [];
    this.learningRate = learningRate;
    this.epochs = epochs;
    this.weights = [];
    this.lossFunction = lossFunction;
  }
  /**
   * Fits the model to the training data
   * @param {number[][]} X - Training data features matrix
   * @param {number[]} y - Training data target values
   */
  fit(X, y) {
    if (X.length === 0 || y.length === 0) {
      throw new DataValidationError("Empty training data");
    }
    if (X.length !== y.length) {
      throw new DataValidationError(
        `Number of samples in X(${X.length}) and y(${y.length}) must match`
      );
    }
    const nSamples = X.length;
    const nFeatures = X[0].length;
    this.weights = Array.from(
      { length: nFeatures + 1 },
      () => Math.random() * 0.01
    );
    for (let epoch = 0; epoch < this.epochs; epoch++) {
      for (let i = 0; i < nSamples; i++) {
        const xi = [1, ...X[i]];
        const yi = y[i];
        const prediction = this.dotProduct(xi, this.weights);
        const error = prediction - yi;
        if (this.lossFunction === "mse") {
          for (let j = 0; j < this.weights.length; j++) {
            this.weights[j] -= this.learningRate * error * xi[j];
          }
        } else if (this.lossFunction === "mae") {
          const sign = error >= 0 ? 1 : -1;
          for (let j = 0; j < this.weights.length; j++) {
            this.weights[j] -= this.learningRate * sign * xi[j];
          }
        } else {
          throw new SGDRegressorError("Invalid loss function");
        }
      }
    }
  }
  /**
   * Predicts the target values for the given data
   * @param {number[][]} X - Data features matrix
   * @returns {number[]} Predicted target values
   */
  predict(X) {
    return X.map((row) => {
      const xi = [1, ...row];
      return this.dotProduct(xi, this.weights);
    });
  }
  /**
   * Evaluates the model's performance
   * @param {number[][]} X - Data features matrix
   * @param {number[]} y - Target values
   * @returns {number} R-squared score
   */
  score(X, y) {
    const predictions = this.predict(X);
    const meanY = y.reduce((sum, val) => sum + val, 0) / y.length;
    const totalSumOfSquares = y.reduce(
      (sum, yi) => sum + Math.pow(yi - meanY, 2),
      0
    );
    const residualSumOfSquares = y.reduce(
      (sum, yi, i) => sum + Math.pow(yi - predictions[i], 2),
      0
    );
    return 1 - residualSumOfSquares / totalSumOfSquares;
  }
  /**
   * Calculates the dot product of two vectors
   * @param {number[]} a - First vector
   * @param {number[]} b - Second vector
   * @returns {number} Dot product
   */
  dotProduct(a, b) {
    return a.reduce((sum, ai, i) => sum + ai * b[i], 0);
  }
};

// src/preprocessing/StandardScaler.ts
var math2 = __toESM(require("mathjs"));
var StandardScaler = class {
  constructor() {
    this.mean = [];
    this.std = [];
  }
  /**
   * Calculate the mean and standard deviation of features
   * @param {number[][]} X - Input data matrix
   * @throws {Error} If input array is empty or has inconsistent feature dimensions
   */
  fit(X) {
    if (!X || !X.length || !X[0].length) {
      throw new Error("Input array cannot be empty");
    }
    const nFeatures = X[0].length;
    const nSamples = X.length;
    this.mean = new Array(nFeatures).fill(0);
    this.std = new Array(nFeatures).fill(0);
    for (let i = 0; i < nFeatures; i++) {
      this.mean[i] = math2.mean(X.map((row) => row[i]));
    }
    for (let i = 0; i < nFeatures; i++) {
      this.std[i] = Number(
        math2.std(
          X.map((row) => row[i]),
          "uncorrected"
        )
      );
      if (this.std[i] === 0) {
        this.std[i] = 1;
      }
    }
  }
  /**
   * Standardize data using the calculated mean and standard deviation
   * @param {number[][]} X - Input data matrix
   * @returns {number[][]} Standardized data matrix
   */
  transform(X) {
    if (!X || !X.length || !X[0].length) {
      throw new Error("Input array cannot be empty");
    }
    if (X[0].length !== this.mean.length) {
      throw new Error("Feature dimensions do not match training set");
    }
    return X.map(
      (row) => row.map((value, i) => (value - this.mean[i]) / this.std[i])
    );
  }
  /**
   * Combine fit and transform operations
   * @param {number[][]} X - Input data matrix
   * @returns {number[][]} Standardized data matrix
   */
  fitTransform(X) {
    this.fit(X);
    return this.transform(X);
  }
};

// src/preprocessing/MinMaxScaler.ts
var math3 = __toESM(require("mathjs"));
var MinMaxScaler = class {
  constructor() {
    this.min = null;
    this.max = null;
  }
  /**
   * Fit the scaler to the data
   * @param {number[][]} X - Input data matrix
   */
  fit(X) {
    if (X.length === 0) {
      throw new DataValidationError("Empty input data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new DataValidationError("Inconsistent number of features");
    }
    this.min = new Array(featureCount).fill(Infinity);
    this.max = new Array(featureCount).fill(-Infinity);
    for (let i = 0; i < X.length; i++) {
      for (let j = 0; j < featureCount; j++) {
        this.min[j] = math3.min(this.min[j], X[i][j]);
        this.max[j] = math3.max(this.max[j], X[i][j]);
      }
    }
  }
  /**
   * Fit the scaler to the data and transform it
   * @param X Input data matrix
   * @returns Scaled data matrix
   */
  fitTransform(X) {
    if (X.length === 0) {
      throw new DataValidationError("Empty input data");
    }
    const featureCount = X[0].length;
    if (!X.every((row) => row.length === featureCount)) {
      throw new DataValidationError("Inconsistent number of features");
    }
    this.min = new Array(featureCount).fill(Infinity);
    this.max = new Array(featureCount).fill(-Infinity);
    for (let i = 0; i < X.length; i++) {
      for (let j = 0; j < featureCount; j++) {
        this.min[j] = math3.min(this.min[j], X[i][j]);
        this.max[j] = math3.max(this.max[j], X[i][j]);
      }
    }
    return this.transform(X);
  }
  /**
   * Transform data using the fitted scaler
   * @param X Input data matrix
   * @returns Scaled data matrix
   */
  transform(X) {
    if (!this.min || !this.max) {
      throw new DataValidationError("Scaler not fitted yet");
    }
    if (X.length === 0) {
      throw new DataValidationError("Empty input data");
    }
    if (!X.every((row) => row.length === this.min.length)) {
      throw new DataValidationError(
        "Number of features must match training data"
      );
    }
    return X.map(
      (row) => row.map((val, j) => {
        const range = math3.subtract(this.max[j], this.min[j]);
        return range === 0 ? 0 : math3.divide(math3.subtract(val, this.min[j]), range);
      })
    );
  }
};

// src/validation/trainTestSplit.ts
function trainTestSplit(testSize = 0.2, randomSeed = 42, ...dataArrays) {
  return dataArrays.map((data) => {
    if (data.length === 0) {
      throw new DataValidationError("Empty data array");
    }
    const shuffled = [...data];
    if (randomSeed) {
      const random = (seed) => {
        let value = seed;
        return () => {
          value = (value * 9301 + 49297) % 233280;
          return value / 233280;
        };
      };
      const rand = random(randomSeed);
      shuffled.sort(() => 0.5 - rand());
    }
    const testSetSize = Math.floor(testSize * data.length);
    return {
      train: shuffled.slice(0, shuffled.length - testSetSize),
      test: shuffled.slice(shuffled.length - testSetSize)
    };
  });
}

// src/index.ts
function createPipelineAndTrain(X, y, testSize = 0.2, modelType = "linear", preprocess = "standard", randomSeed = 42) {
  if (!Array.isArray(X) || !Array.isArray(y)) {
    throw new Error("Input data must be arrays");
  }
  if (X.length === 0 || y.length === 0) {
    throw new Error("Input data cannot be empty");
  }
  if (!X.every(
    (row) => Array.isArray(row) && row.every((val) => typeof val === "number")
  )) {
    throw new Error("Feature matrix X must be a 2D array of numbers");
  }
  if (!y.every((val) => typeof val === "number")) {
    throw new Error("Target vector y must be an array of numbers");
  }
  const [{ train: X_train, test: X_test }, { train: y_train, test: y_test }] = trainTestSplit(testSize, randomSeed, X, y);
  const scaler = preprocess === "standard" ? new StandardScaler() : new MinMaxScaler();
  const X_train_scaled = scaler.fitTransform(X_train);
  const X_test_scaled = scaler.transform(X_test);
  if (X_train_scaled.length === 0) {
    throw new Error("Training data is empty after preprocessing");
  }
  if (X_test_scaled.length === 0) {
    throw new Error("Testing data is empty after preprocessing");
  }
  const model = modelType === "linear" ? new LinearRegression() : new SGDRegressor();
  model.fit(X_train_scaled, y_train);
  const predictions = model.predict(X_test_scaled);
  return {
    model,
    predictions,
    trueValues: y_test,
    scaler,
    testFeaturesRaw: X_test,
    testFeaturesScaled: X_test_scaled,
    trainingScore: model.score ? model.score(X_train_scaled, y_train) : void 0
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  LinearRegression,
  MinMaxScaler,
  SGDRegressor,
  StandardScaler,
  createPipelineAndTrain,
  trainTestSplit
});
